@@ -103,8 +103,8 @@ Supporting OS subroutines required:
 */
 
 #include <_ansi.h>
-#include <newlib.h>
 #include <reent.h>
+#include <newlib.h>
 #include <ctype.h>
 #include <wctype.h>
 #include <stdio.h>
@@ -130,16 +130,11 @@ Supporting OS subroutines required:
    This could be changed in the future should the _ldtoa_r code be
    preferred over _dtoa_r.  */
 #define _NO_LONGDBL
-#if defined WANT_IO_LONG_DBL && (LDBL_MANT_DIG > DBL_MANT_DIG)
+#if defined _WANT_IO_LONG_DOUBLE && (LDBL_MANT_DIG > DBL_MANT_DIG)
 #undef _NO_LONGDBL
 extern _LONG_DOUBLE _strtold _PARAMS((char *s, char **sptr));
 #endif
 
-#define _NO_LONGLONG
-#if defined WANT_PRINTF_LONG_LONG && defined __GNUC__
-# undef _NO_LONGLONG
-#endif
-
 #include "floatio.h"
 
 #if ((MAXEXP+MAXFRACT+3) > MB_LEN_MAX)
@@ -155,6 +150,11 @@ extern _LONG_DOUBLE _strtold _PARAMS((char *s, char **sptr));
 #define	BUF	40
 #endif
 
+#define _NO_LONGLONG
+#if defined _WANT_IO_LONG_LONG && defined __GNUC__
+# undef _NO_LONGLONG
+#endif
+
 /*
  * Flags used during conversion.
  */
@@ -872,6 +872,10 @@ _DEFUN(__svfscanf_r, (rptr, fp, fmt0, ap),
 		    {
 		      base = 16;/* if %i */
 		      flags &= ~PFXOK;
+		      /* We must reset the NZDIGITS and NDIGITS
+		         flags that would have been unset by seeing
+			 the zero that preceded the X or x.  */
+		      flags |= NZDIGITS | NDIGITS;
 		      goto ok;
 		    }
 		  break;
