@@ -1,6 +1,6 @@
 /* cygthread.cc
 
-   Copyright 1998, 1999, 2000, 2001, 2002, 2003 Red Hat, Inc.
+   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2004 Red Hat, Inc.
 
 This software is a copyrighted work licensed under the terms of the
 Cygwin license.  Please consult the file "CYGWIN_LICENSE" for
@@ -15,10 +15,12 @@ details. */
 #include "sync.h"
 #include "cygerrno.h"
 #include "sigproc.h"
+#include "thread.h"
+#include "cygtls.h"
 
 #undef CloseHandle
 
-static cygthread NO_COPY threads[18];
+static cygthread NO_COPY threads[32];
 #define NTHREADS (sizeof (threads) / sizeof (threads[0]))
 
 DWORD NO_COPY cygthread::main_thread_id;
@@ -30,13 +32,6 @@ bool NO_COPY cygthread::exiting;
 DWORD WINAPI
 cygthread::stub (VOID *arg)
 {
-  DECLARE_TLS_STORAGE;
-  exception_list except_entry;
-
-  /* Initialize this thread's ability to respond to things like
-     SIGSEGV or SIGFPE. */
-  init_exceptions (&except_entry);
-
   cygthread *info = (cygthread *) arg;
   if (info->arg == cygself)
     {
@@ -45,7 +40,7 @@ cygthread::stub (VOID *arg)
 	  CloseHandle (info->ev);
 	  CloseHandle (info->thread_sync);
 	}
-    info->ev = info->thread_sync = info->stack_ptr = NULL;
+      info->ev = info->thread_sync = info->stack_ptr = NULL;
     }
   else
     {
@@ -56,6 +51,7 @@ cygthread::stub (VOID *arg)
 	  info->thread_sync = CreateEvent (&sec_none_nih, FALSE, FALSE, NULL);
 	}
     }
+
   while (1)
     {
       if (!info->__name)
@@ -63,7 +59,7 @@ cygthread::stub (VOID *arg)
       else
 	{
 	  if (!info->func || exiting)
-	    ExitThread (0);
+	    return 0;
 
 	  /* Cygwin threads should not call ExitThread directly */
 	  info->func (info->arg == cygself ? info : info->arg);
@@ -91,17 +87,11 @@ cygthread::stub (VOID *arg)
 DWORD WINAPI
 cygthread::simplestub (VOID *arg)
 {
-  DECLARE_TLS_STORAGE;
-  exception_list except_entry;
-
-  /* Initialize this thread's ability to respond to things like
-     SIGSEGV or SIGFPE. */
-  init_exceptions (&except_entry);
-
   cygthread *info = (cygthread *) arg;
   info->stack_ptr = &arg;
+  info->ev = info->h;
   info->func (info->arg == cygself ? info : info->arg);
-  ExitThread (0);
+  return 0;
 }
 
 /* Start things going.  Called from dll_crt0_1. */
@@ -111,24 +101,12 @@ cygthread::init ()
   main_thread_id = GetCurrentThreadId ();
 }
 
-bool
-cygthread::is ()
-{
-  DWORD tid = GetCurrentThreadId ();
-
-  for (DWORD i = 0; i < NTHREADS; i++)
-    if (threads[i].id == tid)
-      return 1;
-
-  return 0;
-}
-
 cygthread *
 cygthread::freerange ()
 {
   cygthread *self = (cygthread *) calloc (1, sizeof (*self));
   self->is_freerange = true;
-  self->ev = self->h;
+  self->inuse = 1;
   return self;
 }
 
@@ -153,6 +131,8 @@ new (size_t)
   char buf[1024];
   if (!GetEnvironmentVariable ("CYGWIN_FREERANGE_NOCHECK", buf, sizeof (buf)))
     api_fatal ("Overflowed cygwin thread pool");
+  else
+    thread_printf ("Overflowed cygwin thread pool");
 #endif
 
   info = freerange ();	/* exhausted thread pool */
