@@ -26,7 +26,7 @@ details. */
 static void __stdcall
 set_dl_error (const char *str)
 {
-  __small_sprintf (_my_tls.locals.dl_buffer, "%s: %E", str);
+  __small_sprintf (_my_tls.locals.dl_buffer, "%s, %E", str);
   _my_tls.locals.dl_error = 1;
 }
 
