@@ -437,12 +437,6 @@ fhandler_base::open_9x (int flags, mode_t mode)
 
   syscall_printf ("(%s, %p)", get_win32_name (), flags);
 
-  if (get_win32_name () == NULL)
-    {
-      set_errno (ENOENT);
-      goto done;
-    }
-
   if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_RDONLY)
     access = GENERIC_READ;
   else if ((flags & (O_RDONLY | O_WRONLY | O_RDWR)) == O_WRONLY)
@@ -486,9 +480,14 @@ fhandler_base::open_9x (int flags, mode_t mode)
     }
 #endif
 
-  /* If mode has no write bits set, we set the R/O attribute. */
-  if (!(mode & (S_IWUSR | S_IWGRP | S_IWOTH)))
-    file_attributes |= FILE_ATTRIBUTE_READONLY;
+  if (flags & O_CREAT && get_device () == FH_FS)
+    {
+      /* If mode has no write bits set, we set the R/O attribute. */
+      if (!(mode & (S_IWUSR | S_IWGRP | S_IWOTH)))
+        file_attributes |= FILE_ATTRIBUTE_READONLY;
+      /* The file attributes are needed for later use in, e.g. fchmod. */
+      pc.file_attributes (file_attributes & FILE_ATTRIBUTE_VALID_SET_FLAGS);
+    } 
 
   x = CreateFile (get_win32_name (), access, shared, &sa, creation_distribution,
 		  file_attributes, 0);
@@ -512,16 +511,16 @@ fhandler_base::open_9x (int flags, mode_t mode)
 	goto done;
    }
 
-  syscall_printf ("%p = CreateFile (%s, %p, %p, %p, %p, %p, 0)",
-		  x, get_win32_name (), access, shared, &sa,
-		  creation_distribution, file_attributes);
-
   set_io_handle (x);
   set_flags (flags, pc.binmode ());
 
   res = 1;
   set_open_status ();
 done:
+  debug_printf ("%p = CreateFile (%s, %p, %p, %p, %p, %p, 0)",
+		x, get_win32_name (), access, shared, &sa,
+		creation_distribution, file_attributes);
+
   syscall_printf ("%d = fhandler_base::open (%s, %p)", res, get_win32_name (),
 		  flags);
   return res;
@@ -534,47 +533,46 @@ fhandler_base::open (int flags, mode_t mode)
   if (!wincap.is_winnt ())
     return fhandler_base::open_9x (flags, mode);
 
+  WCHAR wpath[CYG_MAX_PATH + 10];
+  UNICODE_STRING upath = {0, sizeof (wpath), wpath};
+  pc.get_nt_native_path (upath);
+
+  if (RtlIsDosDeviceName_U (upath.Buffer))
+    return fhandler_base::open_9x (flags, mode);
+
   int res = 0;
   HANDLE x;
   ULONG file_attributes = 0;
-  ULONG shared = wincap.shared ();
+  ULONG shared = (get_major () == DEV_TAPE_MAJOR ? 0 : wincap.shared ());
   ULONG create_disposition;
   ULONG create_options;
   SECURITY_ATTRIBUTES sa = sec_none;
   security_descriptor sd;
-  UNICODE_STRING upath;
-  WCHAR wpath[CYG_MAX_PATH + 10];
   OBJECT_ATTRIBUTES attr;
   IO_STATUS_BLOCK io;
   NTSTATUS status;
 
   syscall_printf ("(%s, %p)", get_win32_name (), flags);
-  if (get_win32_name () == NULL)
-    {
-      set_errno (ENOENT);
-      goto done;
-    }
 
-  InitializeObjectAttributes (&attr, pc.get_nt_native_path (upath, wpath),
-			      OBJ_CASE_INSENSITIVE | OBJ_INHERIT,
+  InitializeObjectAttributes (&attr, &upath, OBJ_CASE_INSENSITIVE | OBJ_INHERIT,
 			      sa.lpSecurityDescriptor, NULL);
 
   switch (query_open ())
     {
       case query_read_control:
-        access = READ_CONTROL;
-        create_options = FILE_OPEN_FOR_BACKUP_INTENT;
-        break;
+	access = READ_CONTROL;
+	create_options = FILE_OPEN_FOR_BACKUP_INTENT;
+	break;
       case query_stat_control:
-        access = READ_CONTROL | FILE_READ_ATTRIBUTES;
-        create_options = FILE_OPEN_FOR_BACKUP_INTENT;
-        break;
+	access = READ_CONTROL | FILE_READ_ATTRIBUTES;
+	create_options = FILE_OPEN_FOR_BACKUP_INTENT;
+	break;
       case query_write_control:
-        access = READ_CONTROL | WRITE_OWNER | WRITE_DAC;
-        create_options = FILE_OPEN_FOR_BACKUP_INTENT | FILE_OPEN_FOR_RECOVERY;
-        break;
+	access = READ_CONTROL | WRITE_OWNER | WRITE_DAC;
+	create_options = FILE_OPEN_FOR_BACKUP_INTENT | FILE_OPEN_FOR_RECOVERY;
+	break;
       default:
-        create_options = 0;
+	create_options = 0;
 	if (get_major () == DEV_TAPE_MAJOR && (flags & O_TEXT))
 	  {
 	    /* O_TEXT is used to indicate write-through on tape devices */
@@ -598,7 +596,7 @@ fhandler_base::open (int flags, mode_t mode)
   if ((flags & O_TRUNC) && ((flags & O_ACCMODE) != O_RDONLY))
     {
       if (flags & O_CREAT)
-	create_disposition = FILE_SUPERSEDE;
+	create_disposition = FILE_OVERWRITE_IF;
       else
 	create_disposition = FILE_OVERWRITE;
     }
@@ -636,34 +634,25 @@ fhandler_base::open (int flags, mode_t mode)
     }
 
   status = NtCreateFile (&x, access, &attr, &io, NULL, file_attributes, shared,
-  			 create_disposition, create_options, NULL, 0);
+			 create_disposition, create_options, NULL, 0);
   if (!NT_SUCCESS (status))
     {
-      if (!wincap.can_open_directories () && pc.isdir ())
-	{
-	  if (flags & (O_CREAT | O_EXCL) == (O_CREAT | O_EXCL))
-	    set_errno (EEXIST);
-	  else if (flags & (O_WRONLY | O_RDWR))
-	    set_errno (EISDIR);
-	  else
-	    nohandle (true);
-	}
       __seterrno_from_win_error (RtlNtStatusToDosError (status));
       if (!nohandle ())
 	goto done;
    }
 
-  syscall_printf ("%x = NtCreateFile "
-  		  "(%p, %x, %s, io, NULL, %x, %x, %x, %x, NULL, 0)",
-		  status, x, access, get_win32_name (), file_attributes, shared,
-		  create_disposition, create_options);
-
   set_io_handle (x);
   set_flags (flags, pc.binmode ());
 
   res = 1;
   set_open_status ();
 done:
+  debug_printf ("%x = NtCreateFile "
+		"(%p, %x, %s, io, NULL, %x, %x, %x, %x, NULL, 0)",
+		status, x, access, get_win32_name (), file_attributes, shared,
+		create_disposition, create_options);
+
   syscall_printf ("%d = fhandler_base::open (%s, %p)", res, get_win32_name (),
 		  flags);
   return res;
@@ -810,52 +799,57 @@ fhandler_base::write (const void *ptr, size_t len)
 	      && current_position >= actual_length + (128 * 1024))
 	    {
 	      /* If the file systemn supports sparse files and the application
-	         is writing after a long seek beyond EOF, convert the file to
+		 is writing after a long seek beyond EOF, convert the file to
 		 a sparse file. */
 	      DWORD dw;
 	      HANDLE h = get_output_handle ();
 	      BOOL r = DeviceIoControl (h, FSCTL_SET_SPARSE, NULL, 0, NULL,
-	      				0, &dw, NULL);
+					0, &dw, NULL);
 	      syscall_printf ("%d = DeviceIoControl(%p, FSCTL_SET_SPARSE, "
 			      "NULL, 0, NULL, 0, &dw, NULL)", r, h);
 	    }
 	  else if (wincap.has_lseek_bug ())
 	    {
 	      /* Oops, this is the bug case - Win95 uses whatever is on the
-	         disk instead of some known (safe) value, so we must seek
+		 disk instead of some known (safe) value, so we must seek
 		 back and fill in the gap with zeros. - DJ
-	         Note: this bug doesn't happen on NT4, even though the
-	         documentation for WriteFile() says that it *may* happen
+		 Note: this bug doesn't happen on NT4, even though the
+		 documentation for WriteFile() says that it *may* happen
 		 on any OS. */
+	      /* Check there is enough space */
+	      if (!SetEndOfFile (get_output_handle ()))
+	        {
+		  __seterrno ();
+		  return -1;
+		}
 	      char zeros[512];
 	      int number_of_zeros_to_write = current_position - actual_length;
 	      memset (zeros, 0, 512);
-	      SetFilePointer (get_output_handle (), 0, NULL, FILE_END);
+	      SetFilePointer (get_output_handle (), actual_length, NULL,
+			      FILE_BEGIN);
 	      while (number_of_zeros_to_write > 0)
 		{
 		  DWORD zeros_this_time = (number_of_zeros_to_write > 512
 					 ? 512 : number_of_zeros_to_write);
 		  DWORD written;
-		  if (!WriteFile (get_output_handle (), zeros, zeros_this_time,
-				  &written, NULL))
+		  DWORD ret = WriteFile (get_output_handle (), zeros,
+					 zeros_this_time, &written, NULL);
+		  if (!ret || written < zeros_this_time)
 		    {
-		      __seterrno ();
-		      if (get_errno () == EPIPE)
-			raise (SIGPIPE);
+		      if (!ret)
+		        {
+			  __seterrno ();
+			  if (get_errno () == EPIPE)
+			    raise (SIGPIPE);
+			}
+		      else
+			set_errno (ENOSPC);
 		      /* This might fail, but it's the best we can hope for */
-		      SetFilePointer (get_output_handle (), current_position, NULL,
-				      FILE_BEGIN);
+		      SetFilePointer (get_output_handle (), current_position,
+				      NULL, FILE_BEGIN);
 		      return -1;
 
 		    }
-		  if (written < zeros_this_time) /* just in case */
-		    {
-		      set_errno (ENOSPC);
-		      /* This might fail, but it's the best we can hope for */
-		      SetFilePointer (get_output_handle (), current_position, NULL,
-				      FILE_BEGIN);
-		      return -1;
-		    }
 		  number_of_zeros_to_write -= written;
 		}
 	    }
@@ -1084,7 +1078,7 @@ fhandler_base::lseek (_off64_t offset, int whence)
   else
     {
       if (poff_high)
-        res += (_off64_t) *poff_high << 32;
+	res += (_off64_t) *poff_high << 32;
 
       /* When next we write(), we will check to see if *this* seek went beyond
 	 the end of the file, and back-seek and fill with zeros if so - DJ */
