@@ -344,7 +344,10 @@ fhandler_socket::dup (fhandler_base *child)
 	{
 	  fhs->fixup_after_fork (hMainProc);
 	  if (fhs->get_io_handle() != (HANDLE) INVALID_SOCKET)
-	    return 0;
+	    {
+	      cygheap->fdtab.inc_need_fixup_before ();
+	      return 0;
+	    }
 	}
       debug_printf ("WSADuplicateSocket failed, trying DuplicateHandle");
     }
@@ -363,6 +366,7 @@ fhandler_socket::dup (fhandler_base *child)
     }
   VerifyHandle (nh);
   fhs->set_io_handle (nh);
+  cygheap->fdtab.inc_need_fixup_before ();
   return 0;
 }
 
@@ -631,7 +635,7 @@ fhandler_socket::accept (struct sockaddr *peer, int *len)
     {
       cygheap_fdnew res_fd;
       if (res_fd >= 0 && fdsock (res_fd, &dev (), res))
-        {
+	{
 	  if (get_addr_family () == AF_LOCAL)
 	    ((fhandler_socket *) res_fd)->set_sun_path (get_sun_path ());
 	  ((fhandler_socket *) res_fd)->set_addr_family (get_addr_family ());
@@ -702,12 +706,12 @@ fhandler_socket::prepare (HANDLE &event, long event_mask)
   closed (false);
   if ((event = WSACreateEvent ()) == WSA_INVALID_EVENT)
     {
-      debug_printf ("WSACreateEvent: %E");
+      debug_printf ("WSACreateEvent, %E");
       return false;
     }
   if (WSAEventSelect (get_socket (), event, event_mask) == SOCKET_ERROR)
     {
-      debug_printf ("WSAEventSelect: %E");
+      debug_printf ("WSAEventSelect, %E");
       return false;
     }
   return true;
@@ -724,14 +728,14 @@ fhandler_socket::wait (HANDLE event, int flags)
   switch (WSAWaitForMultipleEvents (2, ev, FALSE, 10, FALSE))
     {
       case WSA_WAIT_TIMEOUT:
-        ret = 0;
+	ret = 0;
 	break;
       case WSA_WAIT_EVENT_0:
 	if (!WSAEnumNetworkEvents (get_socket (), event, &evts))
 	  {
 	    if (!evts.lNetworkEvents)
 	      {
-	        ret = 0;
+		ret = 0;
 		break;
 	      }
 	    if (evts.lNetworkEvents & FD_OOB)
@@ -777,7 +781,7 @@ fhandler_socket::wait (HANDLE event, int flags)
 	  }
 	break;
       case WSA_WAIT_EVENT_0 + 1:
-        WSASetLastError (WSAEINTR);
+	WSASetLastError (WSAEINTR);
 	break;
       default:
 	WSASetLastError (WSAEFAULT);
@@ -792,12 +796,12 @@ fhandler_socket::release (HANDLE event)
   int last_err = WSAGetLastError ();
   /* KB 168349: NT4 fails if the event parameter is not NULL. */
   WSAEventSelect (get_socket (), NULL, 0);
+  WSACloseEvent (event);
   unsigned long non_block = 0;
   if (ioctlsocket (get_socket (), FIONBIO, &non_block))
     debug_printf ("return to blocking failed: %d", WSAGetLastError ());
   else
     WSASetLastError (last_err);
-  WSACloseEvent (event);
 }
 
 int
@@ -841,14 +845,14 @@ fhandler_socket::recvfrom (void *ptr, size_t len, int flags,
 	  HANDLE evt;
 	  if (prepare (evt, FD_CLOSE | FD_READ | (owner () ? FD_OOB : 0)))
 	    {
-              do
-                {
+	      do
+		{
 		  DWORD lflags = (DWORD) flags;
 		  res = WSARecvFrom (get_socket (), &wsabuf, 1, &ret, &lflags,
 				     from, fromlen, NULL, NULL);
-                }
-              while (res == SOCKET_ERROR
-                     && WSAGetLastError () == WSAEWOULDBLOCK
+		}
+	      while (res == SOCKET_ERROR
+		     && WSAGetLastError () == WSAEWOULDBLOCK
 		     && !closed ()
 		     && !(res = wait (evt, flags)));
 	      release (evt);
@@ -963,13 +967,13 @@ fhandler_socket::recvmsg (struct msghdr *msg, int flags, ssize_t tot)
 	  HANDLE evt;
 	  if (prepare (evt, FD_CLOSE | FD_READ | (owner () ? FD_OOB : 0)))
 	    {
-              do
-                {
+	      do
+		{
 		  DWORD lflags = (DWORD) flags;
 		  res = WSARecvFrom (get_socket (), wsabuf, iovcnt, &ret,
 				     &lflags, from, fromlen, NULL, NULL);
-                }
-              while (res == SOCKET_ERROR
+		}
+	      while (res == SOCKET_ERROR
 		     && WSAGetLastError () == WSAEWOULDBLOCK
 		     && !closed ()
 		     && !(res = wait (evt, flags)));
@@ -1040,7 +1044,7 @@ fhandler_socket::sendto (const void *ptr, size_t len, int flags,
 	  HANDLE evt;
 	  if (prepare (evt, FD_CLOSE | FD_WRITE | (owner () ? FD_OOB : 0)))
 	    {
-	      do 
+	      do
 		{
 		  res = WSASendTo (get_socket (), &wsabuf, 1, &ret,
 				   flags & MSG_WINMASK,
@@ -1048,7 +1052,7 @@ fhandler_socket::sendto (const void *ptr, size_t len, int flags,
 				   tolen, NULL, NULL);
 		}
 	      while (res == SOCKET_ERROR
-	             && WSAGetLastError () == WSAEWOULDBLOCK
+		     && WSAGetLastError () == WSAEWOULDBLOCK
 		     && !(res = wait (evt, 0))
 		     && !closed ());
 	      release (evt);
@@ -1170,17 +1174,17 @@ fhandler_socket::sendmsg (const struct msghdr *msg, int flags, ssize_t tot)
 	  HANDLE evt;
 	  if (prepare (evt, FD_CLOSE | FD_WRITE | (owner () ? FD_OOB : 0)))
 	    {
-              do
-                {
+	      do
+		{
 		  res = WSASendTo (get_socket (), wsabuf, iovcnt,
-		  		   &ret, flags,
+				   &ret, flags,
 				   (struct sockaddr *) msg->msg_name,
 				   msg->msg_namelen, NULL, NULL);
-                }
-              while (res == SOCKET_ERROR
-	             && WSAGetLastError () == WSAEWOULDBLOCK
+		}
+	      while (res == SOCKET_ERROR
+		     && WSAGetLastError () == WSAEWOULDBLOCK
 		     && !(res = wait (evt, 0))
-	             && !closed ());
+		     && !closed ());
 	      release (evt);
 	    }
 	}
@@ -1381,7 +1385,7 @@ fhandler_socket::ioctl (unsigned int cmd, void *p)
     case FIONREAD:
       res = ioctlsocket (get_socket (), FIONREAD, (unsigned long *) p);
       if (res == SOCKET_ERROR)
-        set_winsock_errno ();
+	set_winsock_errno ();
       break;
     default:
       /* We must cancel WSAAsyncSelect (if any) before setting socket to
