@@ -1,6 +1,6 @@
 /* fork.cc
 
-   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+   Copyright 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2004 Red Hat, Inc.
 
 This file is part of Cygwin.
 
@@ -14,8 +14,8 @@ details. */
 #include <stdlib.h>
 #include <stdarg.h>
 #include "security.h"
-#include "fhandler.h"
 #include "path.h"
+#include "fhandler.h"
 #include "dtable.h"
 #include "cygerrno.h"
 #include "sigproc.h"
@@ -129,7 +129,7 @@ fork_copy (PROCESS_INFORMATION &pi, const char *what, ...)
    time out, set errno = EAGAIN and hope the app tries again.  */
 static int
 sync_with_child (PROCESS_INFORMATION &pi, HANDLE subproc_ready,
-		 BOOL hang_child, const char *s)
+		 bool hang_child, const char *s)
 {
   /* We also add the child process handle to the wait. If the child fails
      to initialize (eg. because of a missing dll). Then this
@@ -167,11 +167,11 @@ sync_with_child (PROCESS_INFORMATION &pi, HANDLE subproc_ready,
        */
       if (errcode != STATUS_CONTROL_C_EXIT)
 	{
-	    system_printf ("child %d(%p) died before initialization with status code %p",
-			  pi.dwProcessId, pi.hProcess, errcode);
-	    system_printf ("*** child state %s", s);
+	  system_printf ("child %u(%p) died before initialization with status code %p",
+			 cygwin_pid (pi.dwProcessId), pi.hProcess, errcode);
+	  system_printf ("*** child state %s", s);
 #ifdef DEBUGGING
-	    abort ();
+	  try_to_debug ();
 #endif
 	}
       set_errno (EAGAIN);
@@ -212,13 +212,13 @@ sync_with_parent (const char *s, bool hang_self)
       switch (psync_rc)
 	{
 	case WAIT_TIMEOUT:
-	  api_fatal ("WFSO timed out for %s", s);
+	  api_fatal ("WFSO timed out %s", s);
 	  break;
 	case WAIT_FAILED:
 	  if (GetLastError () == ERROR_INVALID_HANDLE &&
 	      WaitForSingleObject (fork_info->forker_finished, 1) != WAIT_FAILED)
 	    break;
-	  api_fatal ("WFSO failed for %s, fork_finished %p, %E", s,
+	  api_fatal ("WFSO failed %s, fork_finished %p, %E", s,
 		     fork_info->forker_finished);
 	  break;
 	default:
@@ -231,6 +231,7 @@ sync_with_parent (const char *s, bool hang_self)
 static int __stdcall
 fork_child (HANDLE& hParent, dll *&first_dll, bool& load_dlls)
 {
+  extern void fixup_timers_after_fork ();
   debug_printf ("child is running.  pid %d, ppid %d, stack here %p",
 		myself->pid, myself->ppid, __builtin_frame_address (0));
 
@@ -238,10 +239,21 @@ fork_child (HANDLE& hParent, dll *&first_dll, bool& load_dlls)
      Don't call setuid here! The flags are already set. */
   cygheap->user.reimpersonate ();
 
-  sync_with_parent ("after longjmp.", TRUE);
+  sync_with_parent ("after longjmp", true);
   sigproc_printf ("hParent %p, child 1 first_dll %p, load_dlls %d", hParent,
 		  first_dll, load_dlls);
 
+  /* If we've played with the stack, stacksize != 0.  That means that
+     fork() was invoked from other than the main thread.  Make sure that
+     the threadinfo information is properly set up.  */
+  if (fork_info->stacksize)
+    {
+      _main_tls = &_my_tls;
+      _main_tls->init_thread (NULL, NULL);
+      _main_tls->local_clib = *_impure_ptr;
+      _impure_ptr = &_main_tls->local_clib;
+    }
+
 #ifdef DEBUGGING
   char c;
   if (GetEnvironmentVariable ("FORKDEBUG", &c, 1))
@@ -256,16 +268,6 @@ fork_child (HANDLE& hParent, dll *&first_dll, bool& load_dlls)
     }
 #endif
 
-  /* If we've played with the stack, stacksize != 0.  That means that
-     fork() was invoked from other than the main thread.  Make sure that
-     when the "main" thread exits it calls do_exit, like a normal process.
-     Exit with a status code of 0. */
-  if (fork_info->stacksize)
-    {
-      ((DWORD *)fork_info->stackbottom)[-17] = (DWORD)do_exit;
-      ((DWORD *)fork_info->stackbottom)[-15] = (DWORD)0;
-    }
-
   set_file_api_mode (current_codepage);
 
   MALLOC_CHECK;
@@ -284,28 +286,29 @@ fork_child (HANDLE& hParent, dll *&first_dll, bool& load_dlls)
     {
       cygheap->fdtab.fixup_after_fork (hParent);
       ProtectHandleINH (hParent);
-      sync_with_parent ("performed fork fixup.", FALSE);
+      sync_with_parent ("performed fork fixup", false);
     }
   else
     {
       dlls.load_after_fork (hParent, first_dll);
       cygheap->fdtab.fixup_after_fork (hParent);
       ProtectHandleINH (hParent);
-      sync_with_parent ("loaded dlls", TRUE);
+      sync_with_parent ("loaded dlls", true);
     }
 
   ForceCloseHandle (hParent);
   (void) ForceCloseHandle1 (fork_info->subproc_ready, subproc_ready);
   (void) ForceCloseHandle1 (fork_info->forker_finished, forker_finished);
 
+  pinfo_fixup_after_fork ();
+  _my_tls.fixup_after_fork ();
+  sigproc_init ();
+
 #ifdef USE_SERVER
   if (fixup_shms_after_fork ())
     api_fatal ("recreate_shm areas after fork failed");
 #endif
 
-  pinfo_fixup_after_fork ();
-  signal_fixup_after_fork ();
-
   /* Set thread local stuff to zero.  Under Windows 95/98 this is sometimes
      non-zero, for some reason.
      FIXME:  There is a memory leak here after a fork. */
@@ -314,8 +317,10 @@ fork_child (HANDLE& hParent, dll *&first_dll, bool& load_dlls)
       (*t)->set ();
 
   pthread::atforkchild ();
+  fixup_timers_after_fork ();
   wait_for_sigthread ();
   cygbench ("fork-child");
+  cygwin_finished_initializing = true;
   return 0;
 }
 
@@ -452,17 +457,15 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
  out:
 #endif
 
-  char sa_buf[1024];
-  PSECURITY_ATTRIBUTES sec_attribs = sec_user_nih (sa_buf);
   syscall_printf ("CreateProcess (%s, %s, 0, 0, 1, %x, 0, 0, %p, %p)",
 		  myself->progname, myself->progname, c_flags, &si, &pi);
-  __malloc_lock ();
+  bool locked = __malloc_lock ();
   void *newheap;
   newheap = cygheap_setup_for_child (&ch, cygheap->fdtab.need_fixup_before ());
   rc = CreateProcess (myself->progname, /* image to run */
 		      myself->progname, /* what we send in arg0 */
-		      sec_attribs,
-		      sec_attribs,
+		      &sec_none_nih,
+		      &sec_none_nih,
 		      TRUE,	  /* inherit handles from parent */
 		      c_flags,
 		      NULL,	  /* environment filled in later */
@@ -481,6 +484,7 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
       /* Restore impersonation */
       cygheap->user.reimpersonate ();
       cygheap_setup_for_child_cleanup (newheap, &ch, 0);
+      __malloc_unlock ();
       return -1;
     }
 
@@ -496,10 +500,19 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
     }
 
 #ifdef DEBUGGING
-  pinfo forked ((ch.cygpid != 1 ? ch.cygpid : cygwin_pid (pi.dwProcessId)), 1);
+  int forked_pid = ch.cygpid != 1 ? ch.cygpid : cygwin_pid (pi.dwProcessId);
 #else
-  pinfo forked (cygwin_pid (pi.dwProcessId), 1);
+  int forked_pid = cygwin_pid (pi.dwProcessId);
 #endif
+  pinfo forked (forked_pid, 1);
+
+  if (!forked)
+    {
+      syscall_printf ("pinfo failed");
+      if (get_errno () != ENOMEM)
+	set_errno (EAGAIN);
+      goto cleanup;
+    }
 
   /* Initialize things that are done later in dll_crt0_1 that aren't done
      for the forkee.  */
@@ -534,7 +547,7 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
 #endif
 
   /* Wait for subproc to initialize itself. */
-  if (!sync_with_child (pi, subproc_ready, TRUE, "waiting for longjmp"))
+  if (!sync_with_child (pi, subproc_ready, true, "waiting for longjmp"))
     goto cleanup;
 
   /* CHILD IS STOPPED */
@@ -546,15 +559,25 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
 
 
   MALLOC_CHECK;
+  void *impure_beg;
+  void *impure_end;
+  if (&_my_tls == _main_tls)
+    impure_beg = impure_end = NULL;
+  else
+    {
+      impure_beg = _impure_ptr;
+      impure_end = _impure_ptr + 1;
+    }
   rc = fork_copy (pi, "user/cygwin data",
 		  user_data->data_start, user_data->data_end,
 		  user_data->bss_start, user_data->bss_end,
 		  cygheap->user_heap.base, cygheap->user_heap.ptr,
 		  stack_here, ch.stackbottom,
 		  dll_data_start, dll_data_end,
-		  dll_bss_start, dll_bss_end, NULL);
+		  dll_bss_start, dll_bss_end, impure_beg, impure_end, NULL);
 
   __malloc_unlock ();
+  locked = false;
   MALLOC_CHECK;
   if (!rc)
     goto cleanup;
@@ -600,10 +623,13 @@ fork_parent (HANDLE& hParent, dll *&first_dll,
   pi.hThread = NULL;
   pthread::atforkparent ();
 
-  return forked->pid;
+  return forked_pid;
 
 /* Common cleanup code for failure cases */
  cleanup:
+  if (locked)
+    __malloc_unlock ();
+
   /* Remember to de-allocate the fd table. */
   if (pi.hProcess)
     ForceCloseHandle1 (pi.hProcess, childhProc);
@@ -627,7 +653,6 @@ fork ()
   } grouped;
 
   MALLOC_CHECK;
-  sigframe thisframe (mainthread);
 
   debug_printf ("entering");
   grouped.hParent = grouped.first_dll = NULL;
@@ -641,7 +666,6 @@ fork ()
   child_info_fork ch;
 
   int res = setjmp (ch.jmp);
-
   if (res)
     res = fork_child (grouped.hParent, grouped.first_dll, grouped.load_dlls);
   else
@@ -679,9 +703,9 @@ extern "C" int
 vfork ()
 {
 #ifndef NEWVFORK
+  debug_printf ("stub called");
   return fork ();
 #else
-  sigframe thisframe;
   vfork_save *vf = get_vfork_val ();
   char **esp, **pp;
 
@@ -690,34 +714,42 @@ vfork ()
   else if (vf->pid)
     return fork ();
 
+  // FIXME the tls stuff could introduce a signal race if a child process
+  // exits quickly.
   if (!setjmp (vf->j))
     {
       vf->pid = -1;
       __asm__ volatile ("movl %%esp,%0": "=r" (vf->vfork_esp):);
       __asm__ volatile ("movl %%ebp,%0": "=r" (vf->vfork_ebp):);
-      for (pp = (char **)vf->frame, esp = vf->vfork_esp;
+      for (pp = (char **) vf->frame, esp = vf->vfork_esp;
 	   esp <= vf->vfork_ebp + 2; pp++, esp++)
 	*pp = *esp;
       vf->ctty = myself->ctty;
       vf->sid = myself->sid;
       vf->pgid = myself->pgid;
+      cygheap->ctty_on_hold = cygheap->ctty;
+      vf->open_fhs = cygheap->open_fhs;
+      debug_printf ("cygheap->ctty_on_hold %p, cygheap->open_fhs %d", cygheap->ctty_on_hold, cygheap->open_fhs);
       int res = cygheap->fdtab.vfork_child_dup () ? 0 : -1;
       debug_printf ("%d = vfork()", res);
+      _my_tls.call_signal_handler ();	// FIXME: racy
+      vf->tls = _my_tls;
       return res;
     }
 
   vf = get_vfork_val ();
 
-  for (pp = (char **)vf->frame, esp = vf->vfork_esp;
+  for (pp = (char **) vf->frame, esp = vf->vfork_esp;
        esp <= vf->vfork_ebp + 2; pp++, esp++)
     *esp = *pp;
 
-  thisframe.init (mainthread);
   cygheap->fdtab.vfork_parent_restore ();
 
   myself->ctty = vf->ctty;
   myself->sid = vf->sid;
   myself->pgid = vf->pgid;
+  termios_printf ("cygheap->ctty %p, cygheap->ctty_on_hold %p", cygheap->ctty, cygheap->ctty_on_hold);
+  cygheap->open_fhs = vf->open_fhs;
 
   if (vf->pid < 0)
     {
@@ -731,6 +763,9 @@ vfork ()
   vf->pid = 0;
   debug_printf ("exiting vfork, pid %d", pid);
   sig_dispatch_pending ();
+
+  _my_tls.call_signal_handler ();	// FIXME: racy
+  _my_tls = vf->tls;
   return pid;
 #endif
 }
