@@ -82,7 +82,7 @@ class mmap_record
     HANDLE get_handle () const { return mapping_handle_; }
     device& get_device () { return dev; }
     DWORD get_access () const { return access_mode_; }
-    DWORD get_offset () const { return offset_; }
+    _off64_t get_offset () const { return offset_; }
     DWORD get_size () const { return size_to_map_; }
     caddr_t get_address () const { return base_address_; }
 
@@ -168,26 +168,29 @@ mmap_record::alloc_page_map (_off64_t off, DWORD len)
 				       sizeof (DWORD))))
     return false;
 
+  off -= offset_;
+  len = PAGE_CNT (len);
+
   if (wincap.virtual_protect_works_on_shared_pages ())
     {
       DWORD old_prot;
+      DWORD vlen = len * getpagesize ();
 
-      off -= offset_;
-      len = PAGE_CNT (len) * getpagesize ();
       if (off > 0 &&
 	  !VirtualProtect (base_address_, off, PAGE_NOACCESS, &old_prot))
-	syscall_printf ("VirtualProtect(%x,%D) failed: %E", base_address_, off);
-      if (off + len < size_to_map_
-	  && !VirtualProtect (base_address_ + off + len,
-			      size_to_map_ - len - off,
+	syscall_printf ("VirtualProtect(%x,%D) failed, %E", base_address_, off);
+      if (off + vlen < size_to_map_
+	  && !VirtualProtect (base_address_ + off + vlen,
+			      size_to_map_ - vlen - off,
 			      PAGE_NOACCESS, &old_prot))
-	syscall_printf ("VirtualProtect(%x,%D) failed: %E",
-			base_address_ + off + len, size_to_map_ - len - off);
-      off /= getpagesize ();
-      len /= getpagesize ();
-      while (len-- > 0)
-	MAP_SET (off + len);
+	syscall_printf ("VirtualProtect(%x,%D) failed, %E",
+			base_address_ + off + vlen, size_to_map_ - vlen - off);
     }
+
+  off /= getpagesize ();
+
+  while (len-- > 0)
+    MAP_SET (off + len);
   return true;
 }
 
@@ -240,7 +243,7 @@ mmap_record::unmap_pages (caddr_t addr, DWORD len)
   if (wincap.virtual_protect_works_on_shared_pages ()
       && !VirtualProtect (base_address_ + off * getpagesize (),
 			  len * getpagesize (), PAGE_NOACCESS, &old_prot))
-    syscall_printf ("-1 = unmap_pages (): %E");
+    syscall_printf ("-1 = unmap_pages (), %E");
 
   for (; len-- > 0; ++off)
     MAP_CLR (off);
@@ -273,7 +276,7 @@ mmap_record::fixup_page_map ()
     }
 
   for (DWORD off = PAGE_CNT (size_to_map_); off > 0; --off)
-    VirtualProtect (base_address_ + off * getpagesize (), getpagesize (),
+    VirtualProtect (base_address_ + (off - 1) * getpagesize (), getpagesize (),
 		    MAP_ISSET (off - 1) ? prot : PAGE_NOACCESS, &old_prot);
 }
 
@@ -358,11 +361,11 @@ list::search_record (_off64_t off, DWORD len)
 /* Used in munmap() */
 long
 list::search_record (caddr_t addr, DWORD len, caddr_t &m_addr, DWORD &m_len,
-	     _off_t start)
+		     long start)
 {
   caddr_t low, high;
 
-  for (int i = start + 1; i < nrecs; ++i)
+  for (long i = start + 1; i < nrecs; ++i)
     {
       low = (addr >= recs[i].get_address ()) ? addr : recs[i].get_address ();
       high = recs[i].get_address ()
@@ -702,8 +705,7 @@ munmap (void *addr, size_t len)
 extern "C" int
 msync (void *addr, size_t len, int flags)
 {
-  syscall_printf ("addr = %x, len = %u, flags = %x",
-		  addr, len, flags);
+  syscall_printf ("addr %x, len %u, flags %x", addr, len, flags);
 
   /* However, check flags for validity. */
   if ((flags & ~(MS_ASYNC | MS_SYNC | MS_INVALIDATE))
@@ -749,7 +751,7 @@ msync (void *addr, size_t len, int flags)
 	      rec->free_fh (fh);
 
 	      if (ret)
-		syscall_printf ("%d = msync(): %E", ret);
+		syscall_printf ("%d = msync(), %E", ret);
 	      else
 		syscall_printf ("0 = msync()");
 
@@ -786,15 +788,31 @@ mprotect (void *addr, size_t len, int prot)
       return 0;
     }
 
+  /* If write protection is requested, check if the page was
+     originally protected writecopy.  In this case call VirtualProtect
+     requesting PAGE_WRITECOPY, otherwise the VirtualProtect will fail
+     on NT version >= 5.0 */
+  bool writecopy = false;
+  if (prot & PROT_WRITE)
+    {
+      MEMORY_BASIC_INFORMATION mbi;
+      if (VirtualQuery (addr, &mbi, sizeof mbi))
+        {
+	  if (mbi.AllocationProtect == PAGE_WRITECOPY
+	      || mbi.AllocationProtect == PAGE_EXECUTE_WRITECOPY)
+	    writecopy = true;
+	}
+    }
+
   switch (prot)
     {
       case PROT_READ | PROT_WRITE | PROT_EXEC:
       case PROT_WRITE | PROT_EXEC:
-	new_prot = PAGE_EXECUTE_READWRITE;
+	new_prot = writecopy ? PAGE_EXECUTE_WRITECOPY : PAGE_EXECUTE_READWRITE;
 	break;
       case PROT_READ | PROT_WRITE:
       case PROT_WRITE:
-	new_prot = PAGE_READWRITE;
+	new_prot = writecopy ? PAGE_WRITECOPY : PAGE_READWRITE;
 	break;
       case PROT_READ | PROT_EXEC:
 	new_prot = PAGE_EXECUTE_READ;
@@ -817,7 +835,7 @@ mprotect (void *addr, size_t len, int prot)
   if (VirtualProtect (addr, len, new_prot, &old_prot) == 0)
     {
       __seterrno ();
-      syscall_printf ("-1 = mprotect (): %E");
+      syscall_printf ("-1 = mprotect (), %E");
       return -1;
     }
 
