@@ -80,7 +80,7 @@ extract_nt_dom_user (const struct passwd *pw, char *domain, char *user)
 
   domain[0] = 0;
   strlcpy (user, pw->pw_name, UNLEN + 1);
-  debug_printf ("pw_gecos = %x (%s)", pw->pw_gecos, pw->pw_gecos);
+  debug_printf ("pw_gecos %x (%s)", pw->pw_gecos, pw->pw_gecos);
 
   if ((d = strstr (pw->pw_gecos, "U-")) != NULL &&
       (d == pw->pw_gecos || d[-1] == ','))
@@ -156,23 +156,28 @@ str2buf2lsa (LSA_STRING &tgt, char *buf, const char *srcstr)
   memcpy (buf, srcstr, tgt.MaximumLength);
 }
 
+/* The dimension of buf is assumed to be at least strlen(srcstr) + 1,
+   The result will be shorter if the input has multibyte chars */
 void
 str2buf2uni (UNICODE_STRING &tgt, WCHAR *buf, const char *srcstr)
 {
-  tgt.Length = strlen (srcstr) * sizeof (WCHAR);
-  tgt.MaximumLength = tgt.Length + sizeof (WCHAR);
   tgt.Buffer = (PWCHAR) buf;
-  sys_mbstowcs (buf, srcstr, tgt.MaximumLength);
+  tgt.MaximumLength = (strlen (srcstr) + 1) * sizeof (WCHAR);
+  tgt.Length = sys_mbstowcs (buf, srcstr, tgt.MaximumLength / sizeof (WCHAR)) 
+               * sizeof (WCHAR);
+  if (tgt.Length)
+    tgt.Length -= sizeof (WCHAR);
 }
 
 void
-str2buf2uni_cat (UNICODE_STRING &tgt, const char *srcstr)
+str2uni_cat (UNICODE_STRING &tgt, const char *srcstr)
 {
-  DWORD len = strlen (srcstr) * sizeof (WCHAR);
-  sys_mbstowcs (tgt.Buffer + tgt.Length / sizeof (WCHAR), srcstr,
-		len + tgt.MaximumLength);
-  tgt.Length += len;
-  tgt.MaximumLength += len;
+  int len = sys_mbstowcs (tgt.Buffer + tgt.Length / sizeof (WCHAR), srcstr,
+		          (tgt.MaximumLength - tgt.Length) / sizeof (WCHAR));
+  if (len)
+    tgt.Length += (len - 1) * sizeof (WCHAR);
+  else
+    tgt.Length = tgt.MaximumLength = 0;
 }
 
 #if 0				/* unused */
@@ -339,7 +344,7 @@ get_user_groups (WCHAR *wlogonserver, cygsidlist &grp_list, char *user,
 
       sys_wcstombs (dgroup + len, buf[i].grui0_name, GNLEN + 1);
       if (!LookupAccountName (NULL, dgroup, gsid, &glen, domain, &dlen, &use))
-	debug_printf ("LookupAccountName(%s): %E", dgroup);
+	debug_printf ("LookupAccountName(%s), %E", dgroup);
       else if (legal_sid_type (use))
 	grp_list += gsid;
       else
@@ -419,11 +424,11 @@ get_user_local_groups (cygsidlist &grp_list, PSID pusersid)
 	if (!LookupAccountName (NULL, bgroup, gsid, &glen, domain, &dlen, &use))
 	  {
 	    if (GetLastError () != ERROR_NONE_MAPPED)
-	      debug_printf ("LookupAccountName(%s): %E", bgroup);
+	      debug_printf ("LookupAccountName(%s), %E", bgroup);
 	    strcpy (lgroup + llen, bgroup + blen);
 	    if (!LookupAccountName (NULL, lgroup, gsid, &glen,
 				    domain, &dlen, &use))
-	      debug_printf ("LookupAccountName(%s): %E", lgroup);
+	      debug_printf ("LookupAccountName(%s), %E", lgroup);
 	  }
 	if (!legal_sid_type (use))
 	  debug_printf ("Rejecting local %s. use: %d", bgroup + blen, use);
@@ -725,7 +730,7 @@ verify_token (HANDLE token, cygsid &usersid, user_groups &groups, bool *pintern)
       TOKEN_SOURCE ts;
       if (!GetTokenInformation (token, TokenSource,
 				&ts, sizeof ts, &size))
-	debug_printf ("GetTokenInformation(): %E");
+	debug_printf ("GetTokenInformation(), %E");
       else
 	*pintern = intern = !memcmp (ts.SourceName, "Cygwin.1", 8);
     }
@@ -733,7 +738,7 @@ verify_token (HANDLE token, cygsid &usersid, user_groups &groups, bool *pintern)
   cygsid tok_usersid = NO_SID;
   if (!GetTokenInformation (token, TokenUser,
 			    &tok_usersid, sizeof tok_usersid, &size))
-    debug_printf ("GetTokenInformation(): %E");
+    debug_printf ("GetTokenInformation(), %E");
   if (usersid != tok_usersid)
     return false;
 
@@ -746,10 +751,10 @@ verify_token (HANDLE token, cygsid &usersid, user_groups &groups, bool *pintern)
       if (!GetKernelObjectSecurity (token, GROUP_SECURITY_INFORMATION,
 				    (PSECURITY_DESCRIPTOR) sd_buf,
 				    sizeof sd_buf, &size))
-	debug_printf ("GetKernelObjectSecurity(): %E");
+	debug_printf ("GetKernelObjectSecurity(), %E");
       else if (!GetSecurityDescriptorGroup ((PSECURITY_DESCRIPTOR) sd_buf,
 					    (PSID *) &gsid, (BOOL *) &size))
-	debug_printf ("GetSecurityDescriptorGroup(): %E");
+	debug_printf ("GetSecurityDescriptorGroup(), %E");
       if (well_known_null_sid != gsid)
 	return gsid == groups.pgsid;
     }
@@ -760,11 +765,11 @@ verify_token (HANDLE token, cygsid &usersid, user_groups &groups, bool *pintern)
 
   if (!GetTokenInformation (token, TokenGroups, NULL, 0, &size) &&
       GetLastError () != ERROR_INSUFFICIENT_BUFFER)
-    debug_printf ("GetTokenInformation(token, TokenGroups): %E");
+    debug_printf ("GetTokenInformation(token, TokenGroups), %E");
   else if (!(my_grps = (PTOKEN_GROUPS) alloca (size)))
     debug_printf ("alloca (my_grps) failed.");
   else if (!GetTokenInformation (token, TokenGroups, my_grps, size, &size))
-    debug_printf ("GetTokenInformation(my_token, TokenGroups): %E");
+    debug_printf ("GetTokenInformation(my_token, TokenGroups), %E");
   else if (!groups.issetgroups ()) /* setgroups was never called */
     ret = sid_in_token_groups (my_grps, groups.pgsid)
 	  || groups.pgsid == usersid;
@@ -854,7 +859,7 @@ create_token (cygsid &usersid, user_groups &new_groups, struct passwd *pw)
 
   /* Retrieve authentication id and group list from own process. */
   if (!OpenProcessToken (hMainProc, TOKEN_QUERY, &my_token))
-    debug_printf ("OpenProcessToken(my_token): %E");
+    debug_printf ("OpenProcessToken(my_token), %E");
   else
     {
       /* Switching user context to SYSTEM doesn't inherit the authentication
@@ -863,7 +868,7 @@ create_token (cygsid &usersid, user_groups &new_groups, struct passwd *pw)
 	if (!GetTokenInformation (my_token, TokenStatistics,
 				  &stats, sizeof stats, &size))
 	  debug_printf
-	    ("GetTokenInformation(my_token, TokenStatistics): %E");
+	    ("GetTokenInformation(my_token, TokenStatistics), %E");
 	else
 	  auth_luid = stats.AuthenticationId;
 
@@ -871,13 +876,13 @@ create_token (cygsid &usersid, user_groups &new_groups, struct passwd *pw)
 	 some important well known group sids. */
       if (!GetTokenInformation (my_token, TokenGroups, NULL, 0, &size) &&
 	  GetLastError () != ERROR_INSUFFICIENT_BUFFER)
-	debug_printf ("GetTokenInformation(my_token, TokenGroups): %E");
+	debug_printf ("GetTokenInformation(my_token, TokenGroups), %E");
       else if (!(my_tok_gsids = (PTOKEN_GROUPS) malloc (size)))
 	debug_printf ("malloc (my_tok_gsids) failed.");
       else if (!GetTokenInformation (my_token, TokenGroups, my_tok_gsids,
 				     size, &size))
 	{
-	  debug_printf ("GetTokenInformation(my_token, TokenGroups): %E");
+	  debug_printf ("GetTokenInformation(my_token, TokenGroups), %E");
 	  free (my_tok_gsids);
 	  my_tok_gsids = NULL;
 	}
@@ -1077,7 +1082,7 @@ out:
    of the SD on success. Unfortunately NT returns
    0 in `len' on success, while W2K returns the
    correct size!
-   
+
    2003-11-26: Now the function allocates the space needed by itself so
    it knows the real size and returns it in the security_descriptor object.
 */
@@ -1104,11 +1109,11 @@ read_sd (const char *file, security_descriptor &sd)
 			NULL, 0, &len)
       && GetLastError () != ERROR_INSUFFICIENT_BUFFER)
     {
-      debug_printf ("file = %s", file);
+      debug_printf ("file %s", file);
       __seterrno ();
       return -1;
     }
-  debug_printf ("file = %s: len=%d", file, len);
+  debug_printf ("file %s: len %d", file, len);
   if (!sd.malloc (len))
     {
       set_errno (ENOMEM);
@@ -1142,7 +1147,7 @@ write_sd (HANDLE fh, const char *file, security_descriptor &sd)
 				     NULL)) == INVALID_HANDLE_VALUE)
 	break;
       if (fh && (ret = NtSetSecurityObject (fh,
-					    DACL_SECURITY_INFORMATION 
+					    DACL_SECURITY_INFORMATION
 					    | GROUP_SECURITY_INFORMATION
 					    | OWNER_SECURITY_INFORMATION,
 					    sd)) == STATUS_SUCCESS)
@@ -1308,7 +1313,7 @@ get_info_from_sd (PSECURITY_DESCRIPTOR psd, mode_t *attribute,
   else
     get_attribute_from_acl (attribute, acl, owner_sid, group_sid, grp_member);
 
-  syscall_printf ("%sACL = %x, uid %d, gid %d",
+  syscall_printf ("%sACL %x, uid %d, gid %d",
 		  (!acl_exists || !acl)?"NO ":"", *attribute, uid, gid);
   return;
 }
@@ -1768,7 +1773,7 @@ set_security_attribute (int attribute, PSECURITY_ATTRIBUTES psa,
   InitializeSecurityDescriptor ((PSECURITY_DESCRIPTOR)psa->lpSecurityDescriptor,
 				SECURITY_DESCRIPTOR_REVISION);
   psa->lpSecurityDescriptor = alloc_sd (geteuid32 (), getegid32 (),
-  					attribute, sd);
+					attribute, sd);
 }
 
 static int
