@@ -114,11 +114,9 @@ find_exec (const char *name, path_conv& buf, const char *mywinenv,
 
   /* Return the error condition if this is an absolute path or if there
      is no PATH to search. */
-  if (strchr (name, '/') || strchr (name, '\\') ||
-      isdrive (name) ||
-      !(winpath = getwinenv (mywinenv)) ||
-      !(path = winpath->get_native ()) ||
-      *path == '\0')
+  if (has_slash || strchr (name, '\\') || isdrive (name)
+      || !(winpath = getwinenv (mywinenv))
+      || !(path = winpath->get_native ()) || *path == '\0')
     goto errout;
 
   debug_printf ("%s%s", mywinenv, path);
@@ -471,6 +469,13 @@ spawn_guts (const char * prog_arg, const char *const *argv,
 
       debug_printf ("%s is a script", (char *) real_path);
 
+      if (real_path.has_acls () && allow_ntsec
+	  && check_file_access (real_path, X_OK))
+	{
+	  debug_printf ("... but not executable");
+	  break;
+	}
+
       char *pgm, *arg1;
 
       if (buf[0] != '#' || buf[1] != '!')
@@ -861,9 +866,9 @@ spawn_guts (const char * prog_arg, const char *const *argv,
 			     nwait, myself->pid, myself->dwProcessId);
 	      system_printf ("waitbuf[0] %p %d", waitbuf[0],
 			     WaitForSingleObject (waitbuf[0], 0));
-	      system_printf ("waitbuf[1] %p = %d", waitbuf[1],
+	      system_printf ("waitbuf[1] %p %d", waitbuf[1],
 			     WaitForSingleObject (waitbuf[1], 0));
-	      system_printf ("waitbuf[w] %p = %d", waitbuf[2],
+	      system_printf ("waitbuf[w] %p %d", waitbuf[2],
 			     WaitForSingleObject (waitbuf[2], 0));
 	      set_errno (ECHILD);
 	      try_to_debug ();
@@ -874,7 +879,7 @@ spawn_guts (const char * prog_arg, const char *const *argv,
 
       ForceCloseHandle (subproc_ready);
 
-      sigproc_printf ("res = %x", res);
+      sigproc_printf ("res %p", res);
 
       if (res & EXIT_REPARENTING)
 	{
