@@ -707,6 +707,24 @@ link (const char *a, const char *b)
       if (CreateHardLinkA (real_b, real_a, NULL))
 	goto success;
 
+      /* There are two cases to consider:
+	 - The FS doesn't support hard links ==> ERROR_INVALID_FUNCTION
+	   We copy the file.
+	 - CreateHardLinkA is not supported  ==> ERROR_PROC_NOT_FOUND
+	   In that case (<= NT4) we try the old-style method.
+	 Any other error should be taken seriously. */
+      if (GetLastError () == ERROR_INVALID_FUNCTION)
+	{
+	  syscall_printf ("FS doesn't support hard links: Copy file");
+	  goto docopy;
+	}
+      if (GetLastError () != ERROR_PROC_NOT_FOUND)
+        {
+	  syscall_printf ("CreateHardLinkA failed");
+	  __seterrno ();
+	  goto done;
+	}
+
       HANDLE hFileSource;
 
       WIN32_STREAM_ID StreamId;
@@ -717,6 +735,7 @@ link (const char *a, const char *b)
       WCHAR wbuf[CYG_MAX_PATH];
 
       BOOL bSuccess;
+      DWORD write_err;
 
       hFileSource = CreateFile (real_a, FILE_WRITE_ATTRIBUTES,
 				FILE_SHARE_READ | FILE_SHARE_WRITE /*| FILE_SHARE_DELETE*/,
@@ -741,6 +760,7 @@ link (const char *a, const char *b)
 		   StreamId.dwStreamNameSize;
 
       lpContext = NULL;
+      write_err = 0;
       /* Write the WIN32_STREAM_ID */
       bSuccess = BackupWrite (
 	hFileSource,
@@ -767,7 +787,10 @@ link (const char *a, const char *b)
 		);
 
 	  if (!bSuccess)
-	    syscall_printf ("cannot write linkname, %E");
+	    {
+	      write_err = GetLastError ();
+	      syscall_printf ("cannot write linkname, %E");
+	    }
 
 	  /* Free context */
 	  BackupWrite (
@@ -780,12 +803,25 @@ link (const char *a, const char *b)
 	    &lpContext);
 	}
       else
-	syscall_printf ("cannot write streamId, %E");
+	{
+	  write_err = GetLastError ();
+	  syscall_printf ("cannot write streamId, %E");
+	}
 
       CloseHandle (hFileSource);
 
       if (!bSuccess)
-	goto docopy;
+        {
+	  /* Only copy file if FS doesn't support hard links */
+	  if (write_err == ERROR_INVALID_FUNCTION)
+	    {
+	      syscall_printf ("FS doesn't support hard links: Copy file");
+	      goto docopy;
+	    }
+
+	  __seterrno_from_win_error (write_err);
+	  goto done;
+	}
 
     success:
       res = 0;
@@ -916,7 +952,7 @@ chmod (const char *path, mode_t mode)
   fhandler_base *fh;
   if (!(fh = build_fh_name (path, NULL, PC_SYM_FOLLOW, stat_suffixes)))
     goto error;
-      
+
   if (fh->error ())
     {
       debug_printf ("got %d error from build_fh_name", fh->error ());
@@ -1058,10 +1094,10 @@ stat_worker (const char *name, struct __stat64 *buf, int nofollow)
   if (check_null_invalid_struct_errno (buf))
     goto error;
 
-  if (!(fh = build_fh_name (name, NULL, nofollow ? PC_SYM_NOFOLLOW : PC_SYM_FOLLOW, 
+  if (!(fh = build_fh_name (name, NULL, nofollow ? PC_SYM_NOFOLLOW : PC_SYM_FOLLOW,
 			    stat_suffixes)))
     goto error;
-  
+
   if (fh->error ())
     {
       debug_printf ("got %d error from build_fh_name", fh->error ());
@@ -1162,7 +1198,7 @@ access (const char *fn, int flags)
     {
       fhandler_base *fh = build_fh_name (fn, NULL, PC_SYM_FOLLOW, stat_suffixes);
       if (fh)
-        {
+	{
 	  res =  fh->fhaccess (flags);
 	  delete fh;
 	}
@@ -1492,7 +1528,7 @@ ttyname_r (int fd, char *buf, size_t buflen)
       else if (buflen < strlen (cfd->ttyname ()) + 1)
 	ret = ERANGE;
       else
-        strcpy (buf, cfd->ttyname ());
+	strcpy (buf, cfd->ttyname ());
     }
   debug_printf ("returning %d tty: %s", ret, ret ? "NULL" : buf);
   return ret;
@@ -1640,7 +1676,7 @@ setmode (int fd, int mode)
 extern "C" int
 ftruncate64 (int fd, _off64_t length)
 {
-  int res = -1;
+  int res = -1, res_bug = 0;
 
   if (length < 0)
     set_errno (EINVAL);
@@ -1657,10 +1693,13 @@ ftruncate64 (int fd, _off64_t length)
 	      _off64_t prev_loc = cfd->lseek (0, SEEK_CUR);
 
 	      cfd->lseek (length, SEEK_SET);
+	      /* Fill the space with 0, if needed */
+	      if (wincap.has_lseek_bug ())
+		res_bug = cfd->write (&res, 0);
 	      if (!SetEndOfFile (h))
 		__seterrno ();
 	      else
-		res = 0;
+		res = res_bug;
 
 	      /* restore original file pointer location */
 	      cfd->lseek (prev_loc, SEEK_SET);
@@ -1939,10 +1978,9 @@ mknod (const char *_path, mode_t mode, __dev16_t dev)
 }
 
 extern "C" int
-mkfifo (const char *_path, mode_t mode)
+mkfifo (const char *path, mode_t mode)
 {
-  set_errno (ENOSYS);	// FIXME
-  return -1;
+  return mknod32 (path, (mode & ~S_IFMT) | S_IFIFO, 0);
 }
 
 /* seteuid: standards? */
@@ -2010,7 +2048,7 @@ seteuid32 (__uid32_t uid)
       if (!SetTokenInformation (ptok, TokenDefaultDacl,
 				&tdacl, sizeof dacl_buf))
 	debug_printf ("SetTokenInformation"
-		      "(TokenDefaultDacl): %E");
+		      "(TokenDefaultDacl), %E");
     }
 
   /* If no impersonation token is available, try to
@@ -2040,17 +2078,17 @@ seteuid32 (__uid32_t uid)
       if (!SetTokenInformation (new_token, TokenOwner,
 				&usersid, sizeof usersid))
 	debug_printf ("SetTokenInformation(user.token, "
-		      "TokenOwner): %E");
+		      "TokenOwner), %E");
       /* Try setting primary group in token to current group */
       if (!SetTokenInformation (new_token, TokenPrimaryGroup,
 				&groups.pgsid, sizeof (cygsid)))
 	debug_printf ("SetTokenInformation(user.token, "
-		      "TokenPrimaryGroup): %E");
+		      "TokenPrimaryGroup), %E");
       /* Try setting default DACL */
       if (tdacl.DefaultDacl
 	  && !SetTokenInformation (new_token, TokenDefaultDacl,
 				   &tdacl, sizeof (tdacl)))
-	debug_printf ("SetTokenInformation (TokenDefaultDacl): %E");
+	debug_printf ("SetTokenInformation (TokenDefaultDacl), %E");
     }
 
   CloseHandle (ptok);
@@ -2157,22 +2195,22 @@ setegid32 (__gid32_t gid)
 				TokenPrimaryGroup,
 				&gsid, sizeof gsid))
 	debug_printf ("SetTokenInformation(thread, "
-		      "TokenPrimaryGroup): %E");
+		      "TokenPrimaryGroup), %E");
       RevertToSelf ();
     }
   if (!OpenProcessToken (hMainProc, TOKEN_ADJUST_DEFAULT, &ptok))
-    debug_printf ("OpenProcessToken(): %E");
+    debug_printf ("OpenProcessToken(), %E");
   else
     {
       if (!SetTokenInformation (ptok, TokenPrimaryGroup,
 				&gsid, sizeof gsid))
 	debug_printf ("SetTokenInformation(process, "
-		      "TokenPrimaryGroup): %E");
+		      "TokenPrimaryGroup), %E");
       CloseHandle (ptok);
     }
   if (cygheap->user.issetuid ()
       && !ImpersonateLoggedOnUser (cygheap->user.token ()))
-    system_printf ("Impersonating in setegid failed: %E");
+    system_printf ("Impersonating in setegid failed, %E");
   return 0;
 }
 
