@@ -325,19 +325,19 @@ tty::alive (const char *fmt)
   return ev != NULL;
 }
 
-HANDLE 
+HANDLE
 tty::open_output_mutex ()
 {
   return open_mutex (OUTPUT_MUTEX);
-} 
+}
 
 HANDLE
 tty::open_input_mutex ()
 {
   return open_mutex (INPUT_MUTEX);
-} 
+}
 
-HANDLE 
+HANDLE
 tty::open_mutex (const char *mutex)
 {
   char buf[CYG_MAX_PATH];
@@ -353,7 +353,7 @@ tty::create_inuse (const char *fmt)
 
   shared_name (buf, fmt, ntty);
   h = CreateEvent (&sec_all, TRUE, FALSE, buf);
-  termios_printf ("%s = %p", buf, h);
+  termios_printf ("%s %p", buf, h);
   if (!h)
     termios_printf ("couldn't open inuse event, %E", buf);
   return h;
